Kind

Struct Kind 

pub struct Kind { /* private fields */ }
The type (kind) of a given value.

This struct tracks the known states a type can have. By allowing one type to have multiple states, the type definition can be progressively refined.

Implementations

Source

impl Kind

pub fn any() -> Self

The “any” type state.

This state implies all states for the type are valid. There is no known information that can be gleaned from the type.

pub fn json() -> Self

The “json” type state.

This state is similar to any, except that it excludes any types that can’t be represented in a native JSON-type (such as timestamp and regex).

pub const fn bytes() -> Self

The “bytes” type state.

pub const fn integer() -> Self

The “integer” type state.

pub const fn float() -> Self

The “float” type state.

pub const fn boolean() -> Self

The “boolean” type state.

pub const fn timestamp() -> Self

The “timestamp” type state.

pub const fn regex() -> Self

The “regex” type state.

pub const fn null() -> Self

The “null” type state.

pub const fn undefined() -> Self

The “undefined” type state.

pub const fn never() -> Self

The “never” type state.

pub fn array(collection: impl Into<Collection<Index>>) -> Self

The “array” type state.

pub fn object(collection: impl Into<Collection<Field>>) -> Self

The “object” type state.

pub fn any_object() -> Self

An object that can have any fields.

Source

impl Kind

pub const fn or_bytes(self) -> Self

Add the bytes state to the type.

pub const fn or_integer(self) -> Self

Add the integer state to the type.

pub const fn or_float(self) -> Self

Add the float state to the type.

pub const fn or_boolean(self) -> Self

Add the boolean state to the type.

pub const fn or_timestamp(self) -> Self

Add the timestamp state to the type.

pub const fn or_regex(self) -> Self

Add the regex state to the type.

pub const fn or_null(self) -> Self

Add the null state to the type.

pub const fn or_undefined(self) -> Self

Add the undefined state to the type.

pub fn or_array(self, collection: impl Into<Collection<Index>>) -> Self

Add the array state to the type.

pub fn or_object(self, collection: impl Into<Collection<Field>>) -> Self

Add the object state to the type.

Source

impl Kind

pub fn add_bytes(&mut self) -> bool

Add the bytes state to the type.

If the type already included this state, the function returns false.

pub fn add_integer(&mut self) -> bool

Add the integer state to the type.

If the type already included this state, the function returns false.

pub fn add_float(&mut self) -> bool

Add the float state to the type.

If the type already included this state, the function returns false.

pub fn add_boolean(&mut self) -> bool

Add the boolean state to the type.

If the type already included this state, the function returns false.

pub fn add_timestamp(&mut self) -> bool

Add the timestamp state to the type.

If the type already included this state, the function returns false.

pub fn add_regex(&mut self) -> bool

Add the regex state to the type.

If the type already included this state, the function returns false.

pub fn add_null(&mut self) -> bool

Add the null state to the type.

If the type already included this state, the function returns false.

pub fn add_undefined(&mut self) -> bool

Add the null state to the type.

If the type already included this state, the function returns false.

pub fn add_array(&mut self, collection: impl Into<Collection<Index>>) -> bool

Add the array state to the type.

If the type already included this state, the function returns false.

pub fn add_object(&mut self, collection: impl Into<Collection<Field>>) -> bool

Add the object state to the type.

If the type already included this state, the function returns false.

Source

impl Kind

pub fn remove_bytes(&mut self) -> bool

Remove the bytes state from the type.

If the type previously included this state, true is returned.

pub fn remove_integer(&mut self) -> bool

Remove the integer state from the type.

If the type previously included this state, true is returned.

pub fn remove_float(&mut self) -> bool

Remove the float state from the type.

If the type previously included this state, true is returned.

pub fn remove_boolean(&mut self) -> bool

Remove the boolean state from the type.

If the type previously included this state, true is returned.

pub fn remove_timestamp(&mut self) -> bool

Remove the timestamp state from the type.

If the type previously included this state, true is returned.

pub fn remove_regex(&mut self) -> bool

Remove the regex state from the type.

If the type previously included this state, true is returned.

pub fn remove_null(&mut self) -> bool

Remove the null state from the type.

If the type previously included this state, true is returned.

pub fn remove_undefined(&mut self) -> bool

Remove the undefined state from the type.

If the type previously included this state, true is returned.

pub fn remove_array(&mut self) -> bool

Remove the array state from the type.

If the type previously included this state, true is returned.

pub fn remove_object(&mut self) -> bool

Remove the object state from the type.

If the type previously included this state, true is returned.

Source

impl Kind

pub fn without_undefined(&self) -> Self

Remove the undefined state from the type, and return it.

pub fn without_array(&self) -> Self

Remove the array state from the type, and return it.

pub fn without_object(&self) -> Self

Remove the object state from the type, and return it.

Source

impl Kind

pub const fn is_any(&self) -> bool

Returns true if all type states are valid.

That is, this method only returns true if the object matches all of the known types.

pub const fn is_json(&self) -> bool

Returns true if the JSON type states are valid.

pub const fn is_collection(&self) -> bool

Returns true if only collection type states are valid.

pub const fn is_bytes(&self) -> bool

Returns true if the type is bytes.

pub const fn is_integer(&self) -> bool

Returns true if the type is integer.

pub const fn is_never(&self) -> bool

Returns true if the type is never.

pub const fn is_float(&self) -> bool

Returns true if the type is float.

pub const fn is_boolean(&self) -> bool

Returns true if the type is boolean.

pub const fn is_timestamp(&self) -> bool

Returns true if the type is timestamp.

pub const fn is_regex(&self) -> bool

Returns true if the type is regex.

pub const fn is_null(&self) -> bool

Returns true if the type is null.

pub const fn is_undefined(&self) -> bool

Returns true if the type is undefined.

pub const fn is_array(&self) -> bool

Returns true if the type is array.

pub const fn is_object(&self) -> bool

Returns true if the type is object.

pub const fn is_exact(&self) -> bool

Returns true if at most one type is set.

pub fn is_superset(&self, other: &Self) -> Result<(), OwnedValuePath>

Check if self is a superset of other.

Meaning, if other has a type defined as valid, then self needs to have it defined as valid as well.

Collection types are recursively checked (meaning, known fields in self also need to be a superset of other.

Errors

If the type is not a superset, a path to one field that doesn’t match is returned. This is mostly useful for debugging.

pub const fn intersects(&self, other: &Self) -> bool

Check if self intersects other.

Returns true if there are type states common to both self and other.

Source

impl Kind

pub const fn contains_bytes(&self) -> bool

Returns true if the type is at least bytes.

pub const fn contains_integer(&self) -> bool

Returns true if the type is at least integer.

pub const fn contains_float(&self) -> bool

Returns true if the type is at least float.

pub const fn contains_boolean(&self) -> bool

Returns true if the type is at least boolean.

pub const fn contains_timestamp(&self) -> bool

Returns true if the type is at least timestamp.

pub const fn contains_regex(&self) -> bool

Returns true if the type is at least regex.

pub const fn contains_null(&self) -> bool

Returns true if the type is at least null.

pub const fn contains_undefined(&self) -> bool

Returns true if the type is at least undefined.

pub const fn contains_any_defined(&self) -> bool

Returns true if the type can be any type other than undefined

pub const fn contains_array(&self) -> bool

Returns true if the type is at least array.

pub const fn contains_object(&self) -> bool

Returns true if the type is at least object.

pub const fn contains_primitive(&self) -> bool

Returns true if the type contains at least one non-collection type.

Source

impl Kind

pub const fn as_object(&self) -> Option<&Collection<Field>>

Get the inner object collection.

This returns None if the type is not known to be an object.

pub fn as_object_mut(&mut self) -> Option<&mut Collection<Field>>

Get a mutable reference to the inner object collection.

This returns None if the type is not known to be an object.

pub fn into_object(self) -> Option<Collection<Field>>

Take an object Collection type out of the Kind.

This returns None if the type is not known to be an object.

pub const fn as_array(&self) -> Option<&Collection<Index>>

Get the inner array collection.

This returns None if the type is not known to be an array.

pub fn as_array_mut(&mut self) -> Option<&mut Collection<Index>>

Get a mutable reference to the inner array collection.

This returns None if the type is not known to be an array.

pub fn into_array(self) -> Option<Collection<Index>>

Take an array Collection type out of the Kind.

This returns None if the type is not known to be an array.

pub fn to_primitives(self) -> Self

Returns Kind, with non-primitive states removed.

That is, it returns self, but removes the object and array states.

pub fn upgrade_undefined(self) -> Self

VRL has an interesting property where accessing an undefined value “upgrades” it to a “null” value. This should be used in places those implicit upgrades can occur.

Source

impl Kind

pub fn get<'a>(&self, path: impl ValuePath<'a>) -> Self

Returns the type of a value that is retrieved from a certain path.

This has the same behavior as Value::get, including the implicit conversion of “undefined” to “null.

If you want the type without the implicit type conversion, use Kind::at_path instead.

pub fn at_path<'a>(&self, path: impl ValuePath<'a>) -> Self

This retrieves the Kind at a given path. There is a subtle difference between this and Kind::get where this function does not convert undefined to null. It is viewing the type of a value in-place, before it is retrieved.

Source

impl Kind

pub fn insert<'a>(&mut self, path: impl ValuePath<'a>, kind: Self)

Insert the Kind at the given path within self. This has the same behavior as Value::insert.

pub fn set_at_path<'a>(&mut self, path: impl ValuePath<'a>, kind: Self)

Set the Kind at the given path within self. There is a subtle difference between this and Kind::insert where this function does not convert undefined to null.

pub fn insert_recursive<'a, 'b>( &'a mut self, iter: impl Iterator<Item = BorrowedSegment<'b>> + Clone, kind: Self, )

Insert the Kind at the given path within self. This has the same behavior as Value::insert.

Panics

Object/Array not present in self.

Source

impl Kind

pub fn remove(&mut self, path: &OwnedValuePath, prune: bool) -> Self

Removes the Kind at the given path within self. This has the same behavior as Value::remove.

Source

impl Kind

pub fn debug_info(&self) -> ObjectMap

Returns a tree representation of Kind, in a more human readable format. This is for debugging / development purposes only.

Source

impl Kind

pub fn merge(&mut self, other: Self, strategy: Strategy)

Merge other into self, using the provided Strategy.

pub fn union(&self, other: Self) -> Self

Returns the union of self and other.

pub fn merge_keep(&mut self, other: Self, overwrite: bool)

Merge other into self, optionally overwriting on conflicts.

Source

impl Kind

pub fn canonicalize(&self) -> Self

Returns a Kind type in a standard / simple representation.