pub struct Collection<T: Ord> { /* private fields */ }Expand description
The kinds of a collection (e.g. array or object).
A collection contains one or more kinds for known positions within the collection (e.g. indices or fields), and contains a global “unknown” state that applies to all unknown paths.
Implementations
Sourceimpl Collection<Index>
impl Collection<Index>
Sourcepub fn largest_known_index(&self) -> Option<usize>
pub fn largest_known_index(&self) -> Option<usize>
Returns the largest known index, or None if no known indices exist.
Sourcepub fn get_positive_index(&self, index: isize) -> Option<usize>
pub fn get_positive_index(&self, index: isize) -> Option<usize>
Converts a negative index to a positive index (only if the exact positive index is known).
Sourcepub fn min_length(&self) -> usize
pub fn min_length(&self) -> usize
The minimum possible length an array could be given the type information.
Sourcepub fn exact_length(&self) -> Option<usize>
pub fn exact_length(&self) -> Option<usize>
The exact length of the array, if it can be proven. Otherwise, None.
Sourcepub fn remove_shift(&mut self, index: usize)
pub fn remove_shift(&mut self, index: usize)
Removes the known value at the given index and shifts the elements to the left.
Sourceimpl<T: Ord + Clone> Collection<T>
impl<T: Ord + Clone> Collection<T>
Sourcepub fn from_parts(known: BTreeMap<T, Kind>, unknown: impl Into<Kind>) -> Self
pub fn from_parts(known: BTreeMap<T, Kind>, unknown: impl Into<Kind>) -> Self
Create a new collection from its parts.
Sourcepub fn from_unknown(unknown: impl Into<Kind>) -> Self
pub fn from_unknown(unknown: impl Into<Kind>) -> Self
Create a new collection with a defined “unknown fields” value, and no known fields.
Sourcepub fn empty() -> Self
pub fn empty() -> Self
Create a collection kind of which there are no known and no unknown kinds.
Sourcepub fn any() -> Self
pub fn any() -> Self
Create a collection kind of which the encapsulated values can be any kind.
Sourcepub fn json() -> Self
pub fn json() -> Self
Create a collection kind of which the encapsulated values can be any JSON-compatible kind.
Sourcepub fn is_any(&self) -> bool
pub fn is_any(&self) -> bool
Check if the collection fields can be of any kind.
This returns false if at least one field kind is known.
Sourcepub fn known(&self) -> &BTreeMap<T, Kind>
pub fn known(&self) -> &BTreeMap<T, Kind>
Get a reference to the “known” elements in the collection.
Sourcepub fn known_mut(&mut self) -> &mut BTreeMap<T, Kind>
pub fn known_mut(&mut self) -> &mut BTreeMap<T, Kind>
Get a mutable reference to the “known” elements in the collection.
Sourcepub fn unknown_kind(&self) -> Kind
pub fn unknown_kind(&self) -> Kind
Gets the type of “unknown” elements in the collection. The returned type will always have “undefined” included.
Sourcepub fn is_unknown_exact(&self) -> bool
pub fn is_unknown_exact(&self) -> bool
Returns true if the unknown variant is “Exact” (vs “Infinite”). This can be used to determine when to stop recursing into an unknown kind. Once the unknown is infinite, this will return false and all unknowns after that will return the same kind.
Sourcepub fn set_unknown(&mut self, unknown: impl Into<Kind>)
pub fn set_unknown(&mut self, unknown: impl Into<Kind>)
Set all “unknown” collection elements to the given kind.
Sourcepub fn with_unknown(self, unknown: impl Into<Kind>) -> Self
pub fn with_unknown(self, unknown: impl Into<Kind>) -> Self
Returns a new collection with the unknown set.
Sourcepub fn with_known(self, key: impl Into<T>, kind: Kind) -> Self
pub fn with_known(self, key: impl Into<T>, kind: Kind) -> Self
Returns a new collection that includes the known key.
Sourcepub fn anonymize(&mut self)
pub fn anonymize(&mut self)
Given a collection of known and unknown types, merge the known types with the unknown type, and remove a reference to the known types.
That is, given an object with field “foo” as integer, “bar” as bytes and unknown fields as timestamp, after calling this function, the object has no known fields, and all unknown fields are marked as either an integer, bytes or timestamp.
Recursively known fields are left untouched. For example, an object with a field “foo” that has an object with a field “bar” results in a collection of which any field can have an object that has a field “bar”.
Sourcepub fn merge(&mut self, other: Self, overwrite: bool)
pub fn merge(&mut self, other: Self, overwrite: bool)
Merge the other collection into self.
The following merge strategies are applied.
For known fields:
-
If a field exists in both collections, their
Kinds are merged, or theotherfields are used (depending on the configuredStrategy). -
If a field exists in one but not the other, the field is merged with the “unknown” of the other if it exists, or just the field is used otherwise.
For unknown fields or indices:
- Both
Unknowns are merged, similar to merging twoKinds.
Sourcepub fn reduced_kind(&self) -> Kind
pub fn reduced_kind(&self) -> Kind
Return the reduced Kind of the items within the collection.
This only returns the type of defined values in the collection. Accessing
a non-existing value can return undefined which is not added to the type here.
Sourceimpl<T: Ord + Clone + CollectionKey> Collection<T>
impl<T: Ord + Clone + CollectionKey> Collection<T>
Sourcepub fn is_superset(&self, other: &Self) -> Result<(), OwnedValuePath>
pub fn is_superset(&self, other: &Self) -> Result<(), OwnedValuePath>
Check if self is a superset of other.
Meaning, for all known fields in other, if the field also exists in self, then its type
needs to be a subset of self, otherwise its type needs to be a subset of self’s
unknown.
If self has known fields not defined in other, then other’s unknown must be
a superset of those fields defined in self.
Additionally, other’s unknown type needs to be a subset of self’s.
Errors
If the type is not a superset, a path to one field that doesn’t match is returned. This is mostly useful for debugging.