pub struct Unknown(/* private fields */);Expand description
The type-state of “unknown” values in a collection.
That is, given a collection, it can have a set of “known” value types (e.g. we know the object
collection has a field .foo with a type integer), but also a singular “unknown” value type
(e.g. the array collection has an integer value at index 0, and is 3 values in size. We don’t
know the exact values for indices 1 and 2, but we do know that it has to be the type defined by
Unknown).
“unknown” values can either be “undefined” or the “unknown” type. For example, an array with an infinite unknown of “integer” doesn’t imply that every index contains an array. Rather, it says every index contains an “integer” or is “undefined”.
Implementations
Sourceimpl Unknown
impl Unknown
Sourcepub fn to_kind(&self) -> Kind
pub fn to_kind(&self) -> Kind
Get the Kind stored in this Unknown.
This represents the kind of any type not “known”.
It will always include “undefined”, since unknown
values are not guaranteed to exist.
Sourcepub fn to_existing_kind(&self) -> Kind
pub fn to_existing_kind(&self) -> Kind
Get the Kind stored in this Unknown.
This represents the kind of any EXISTING type not “known”.
This function assumes the type you are accessing actually exists.
If it’s an optional field, to_kind should be used instead.
This will never have “undefined” as part of the type